#include <stdlib.h>
#include <stdio.h>
#include <pthread.h>
#include "basic.h"
#include "poly.h"

#define NTASKS 1024

/* Boxed count_t */

typedef struct {
  count_t c ;
} val_t ;

void free_val(val_t *p) { free(p) ; }

val_t *alloc_val(count_t c) {
  val_t *r = malloc_check(sizeof(*r)) ;
  r->c = c ;
  return r ;
}

/* Worker */

static pthread_t th[NTASKS] ;
static int th_next ;


void *run_stub(void *_z) {
  subtask_t *z = _z ;
  if (verbose) fprintf(stderr,"START: %i\n",z->id) ;
  count_t r = run_subtask(z) ;
  if (verbose) fprintf(stderr,"OVER: %i\n",z->id) ;
  free_arg(z) ; // Optional
  return alloc_val(r) ;
}

void emit_thread(subtask_t *z) {
  if (th_next >= NTASKS) {
    fprintf(stderr, "Too many substasks\n") ;
    exit(2) ;
  }
 // Yes, memory for *z must survive
  create_thread(&th[th_next],run_stub,copy_arg(z)) ;
  th_next++ ;
}
count_t run(int n, int depth) {
  count_t sum = 0 ;

  // Fork
  th_next = 0 ;
  int ntasks = generate_subtasks(n,depth,emit_thread) ;
  if (verbose) fprintf(stderr,"Subtasks: %i\n",ntasks) ;
  // Join
  for (int k = 0 ; k < ntasks ; k++) {
    val_t *r = (val_t *)join_thread(&th[k]) ;
    sum += r->c ;
    free_val(r) ; // optional
  }

  return sum ;
}

void usage(char *pgm)
{
  fprintf(stderr,"usage: %s [-v|-g<int>]* [n]\n",pgm);
  exit(-1);
}

int main(int argc, char **argv) {
  char *arg=0;
  char *pgm = argv[0];
  int n = 20 ;
  int gen_now = 15 ;

  while (--argc) {
    arg = *++argv;
    if (*arg == '-') {
      arg++ ;
       if (*arg == 'g') {
        if (arg[1]) {
          gen_now += atoi(arg+1) ;
        } else {
          usage(pgm) ;
        }
       } else if (*arg == 'v') {
         verbose++;
       } else {
         usage(pgm);
       }
    } else break ;      
  }
  if (argc >= 1) n = atoi(arg);
  if (n > NMAX) {
      fprintf(stderr,"argument must be <= %d\n",NMAX);
      exit(2);
  }
  count_t r = run(n,gen_now) ;
  printf("%" PRC "\n",r) ;
  return 0 ;
}
