#include <stdlib.h>
#include <stdio.h>
#include "basic-c11.h"

/*************/
/* Utilities */
/*************/

static void *malloc_check(size_t sz) {
  if (sz == 0) return NULL ;
  void *p = malloc(sz) ;
  if (!p) {
    perror("malloc") ;
    exit(2) ;
  }
  return p ;
}

static void exit_cmd(char *cmd) {
  perror(cmd) ;
  exit(EXIT_FAILURE) ;
}

/***********/
/* Threads */
/***********/

void create_thread(thrd_t *th, int (*f)(void *),void *a) {
  int st = thrd_create(th,f,a) ;
  if (st != thrd_success) exit_cmd("thrd_create") ;
}

int join_thread(thrd_t th) {
  int r ;
  int st = thrd_join(th,&r) ;
  if (st != thrd_success) exit_cmd("thrd_join") ;
  return r ;
}

/***************/
/* C11 Mutexes */
/***************/

mtx_t *alloc_mutex(void) {
  mtx_t *r = malloc_check(sizeof(*r)) ;
  int st = mtx_init(r,mtx_plain) ;
  if (st != thrd_success) exit_cmd("mtx_init") ;
  return r ;
}

void free_mutex(mtx_t *p) {
  mtx_destroy(p) ;
  free(p) ;
}

void lock_mutex(mtx_t *p) {
  int st = mtx_lock(p) ;
  if (st != thrd_success) exit_cmd("mtx_lock") ;
}

void unlock_mutex(mtx_t *p) {
  int st = mtx_unlock(p) ;
  if (st != thrd_success) exit_cmd("mtx_unlock") ;
}

